/* IMPORT */
import parse from './parse.js';
/* HELPERS */
const STAR_RE = /^\*+$/;
const STATIC_RE = /^[ a-zA-Z0-9/._-]*$/;
/* MAIN */
const matcher = (glob, caseSensitive) => {
    /* STAR SPECIAL-CASE */
    if (STAR_RE.test(glob)) {
        return () => true;
    }
    /* STATIC SPECIAL-CASE */
    if (STATIC_RE.test(glob)) {
        if (caseSensitive) {
            return (segment) => segment === glob;
        }
        else {
            const globLowerCase = glob.toLowerCase();
            const globLength = globLowerCase.length;
            return (segment) => segment.length === globLength && segment.toLowerCase() === globLowerCase;
        }
    }
    /* GENERAL CASE */
    const re = parse(glob, caseSensitive);
    return (segment) => re.test(segment);
};
/* EXPORT */
export default matcher;
